@@ -26,7 +26,9 @@ use crate::volatile_memory::VolatileSlice;
 ///
 /// A type `T` is `ByteValued` if and only if it can be initialized by reading its contents from a
 /// byte array.  This is generally true for all plain-old-data structs.  It is notably not true for
-/// any type that includes a reference.
+/// any type that includes a reference. It is generally also not safe for non-packed structs, as
+/// compiler-inserted padding is considered uninitialized memory, and thus reads/writing it will
+/// cause undefined behavior.
 ///
 /// Implementing this trait guarantees that it is safe to instantiate the struct with random data.
 pub unsafe trait ByteValued: Copy + Default + Send + Sync {
@@ -120,7 +122,7 @@ pub unsafe trait ByteValued: Copy + Default + Send + Sync {
     /// be no other accesses).
     fn as_bytes(&mut self) -> VolatileSlice {
         // SAFETY: This is safe because the lifetime is the same as self
-        unsafe { VolatileSlice::new(self as *mut Self as usize as *mut _, size_of::<Self>()) }
+        unsafe { VolatileSlice::new(self as *mut Self as *mut _, size_of::<Self>()) }
     }
 }
 
@@ -153,11 +155,13 @@ byte_valued_type!(u8);
 byte_valued_type!(u16);
 byte_valued_type!(u32);
 byte_valued_type!(u64);
+byte_valued_type!(u128);
 byte_valued_type!(usize);
 byte_valued_type!(i8);
 byte_valued_type!(i16);
 byte_valued_type!(i32);
 byte_valued_type!(i64);
+byte_valued_type!(i128);
 byte_valued_type!(isize);
 
 /// A trait used to identify types which can be accessed atomically by proxy.
@@ -334,9 +338,9 @@ pub(crate) mod tests {
     #![allow(clippy::undocumented_unsafe_blocks)]
     use super::*;
 
+    use std::cell::RefCell;
     use std::fmt::Debug;
     use std::mem::align_of;
-    use std::slice;
 
     // Helper method to test atomic accesses for a given `b: Bytes` that's supposed to be
     // zero-initialized.
@@ -360,7 +364,7 @@ pub(crate) mod tests {
     where
         T: ByteValued + PartialEq + Debug + Default,
     {
-        let mut data = [0u8; 32];
+        let mut data = [0u8; 48];
         let pre_len = {
             let (pre, _, _) = unsafe { data.align_to::<T>() };
             pre.len()
@@ -375,7 +379,7 @@ pub(crate) mod tests {
                 assert_eq!(val.as_mut_slice(), aligned_data);
             }
         }
-        for i in 1..size_of::<T>() {
+        for i in 1..size_of::<T>().min(align_of::<T>()) {
             let begin = pre_len + i;
             let end = begin + size_of::<T>();
             let unaligned_data = &mut data[begin..end];
@@ -399,24 +403,26 @@ pub(crate) mod tests {
         check_byte_valued_type::<u16>();
         check_byte_valued_type::<u32>();
         check_byte_valued_type::<u64>();
+        check_byte_valued_type::<u128>();
         check_byte_valued_type::<usize>();
         check_byte_valued_type::<i8>();
         check_byte_valued_type::<i16>();
         check_byte_valued_type::<i32>();
         check_byte_valued_type::<i64>();
+        check_byte_valued_type::<i128>();
         check_byte_valued_type::<isize>();
     }
 
     pub const MOCK_BYTES_CONTAINER_SIZE: usize = 10;
 
     pub struct MockBytesContainer {
-        container: [u8; MOCK_BYTES_CONTAINER_SIZE],
+        container: RefCell<[u8; MOCK_BYTES_CONTAINER_SIZE]>,
     }
 
     impl MockBytesContainer {
         pub fn new() -> Self {
             MockBytesContainer {
-                container: [0; MOCK_BYTES_CONTAINER_SIZE],
+                container: RefCell::new([0; MOCK_BYTES_CONTAINER_SIZE]),
             }
         }
 
@@ -443,10 +449,8 @@ pub(crate) mod tests {
         fn write_slice(&self, buf: &[u8], addr: usize) -> Result<(), Self::E> {
             self.validate_slice_op(buf, addr)?;
 
-            // We need to get a mut reference to `self.container`.
-            let container_ptr = self.container[addr..].as_ptr() as usize as *mut u8;
-            let container = unsafe { slice::from_raw_parts_mut(container_ptr, buf.len()) };
-            container.copy_from_slice(buf);
+            let mut container = self.container.borrow_mut();
+            container[addr..addr + buf.len()].copy_from_slice(buf);
 
             Ok(())
         }
@@ -454,7 +458,8 @@ pub(crate) mod tests {
         fn read_slice(&self, buf: &mut [u8], addr: usize) -> Result<(), Self::E> {
             self.validate_slice_op(buf, addr)?;
 
-            buf.copy_from_slice(&self.container[addr..buf.len()]);
+            let container = self.container.borrow();
+            buf.copy_from_slice(&container[addr..addr + buf.len()]);
 
             Ok(())
         }
