@@ -42,7 +42,6 @@
 //! `VolatileSlice`, `VolatileRef`, or `VolatileArrayRef`.
 
 use std::convert::From;
-use std::fmt::{self, Display};
 use std::fs::File;
 use std::io::{self, Read, Write};
 use std::ops::{BitAnd, BitOr, Deref};
@@ -59,17 +58,22 @@ static MAX_ACCESS_CHUNK: usize = 4096;
 
 /// Errors associated with handling guest memory accesses.
 #[allow(missing_docs)]
-#[derive(Debug)]
+#[derive(Debug, thiserror::Error)]
 pub enum Error {
     /// Failure in finding a guest address in any memory regions mapped by this guest.
+    #[error("Guest memory error: invalid guest address {}",.0.raw_value())]
     InvalidGuestAddress(GuestAddress),
     /// Couldn't read/write from the given source.
+    #[error("Guest memory error: {0}")]
     IOError(io::Error),
     /// Incomplete read or write.
+    #[error("Guest memory error: only used {completed} bytes in {expected} long buffer")]
     PartialBuffer { expected: usize, completed: usize },
     /// Requested backend address is out of range.
+    #[error("Guest memory error: invalid backend address")]
     InvalidBackendAddress,
     /// Host virtual address not available.
+    #[error("Guest memory error: host virtual address not available")]
     HostAddressNotAvailable,
 }
 
@@ -95,30 +99,6 @@ impl From<volatile_memory::Error> for Error {
 /// Result of guest memory operations.
 pub type Result<T> = std::result::Result<T, Error>;
 
-impl std::error::Error for Error {}
-
-impl Display for Error {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        write!(f, "Guest memory error: ")?;
-        match self {
-            Error::InvalidGuestAddress(addr) => {
-                write!(f, "invalid guest address {}", addr.raw_value())
-            }
-            Error::IOError(error) => write!(f, "{}", error),
-            Error::PartialBuffer {
-                expected,
-                completed,
-            } => write!(
-                f,
-                "only used {} bytes in {} long buffer",
-                completed, expected,
-            ),
-            Error::InvalidBackendAddress => write!(f, "invalid backend address"),
-            Error::HostAddressNotAvailable => write!(f, "host virtual address not available"),
-        }
-    }
-}
-
 /// Represents a guest physical address (GPA).
 ///
 /// # Notes:
@@ -250,6 +230,8 @@ pub trait GuestMemoryRegion: Bytes<MemoryRegionAddress, E = Error> {
     /// # Safety
     ///
     /// Unsafe because of possible aliasing.
+    #[deprecated = "It is impossible to use this function for accessing memory of a running virtual \
+    machine without violating aliasing rules "]
     unsafe fn as_slice(&self) -> Option<&[u8]> {
         None
     }
@@ -263,6 +245,8 @@ pub trait GuestMemoryRegion: Bytes<MemoryRegionAddress, E = Error> {
     /// Unsafe because of possible aliasing. Mutable accesses performed through the
     /// returned slice are not visible to the dirty bitmap tracking functionality of
     /// the region, and must be manually recorded using the associated bitmap object.
+    #[deprecated = "It is impossible to use this function for accessing memory of a running virtual \
+    machine without violating aliasing rules "]
     unsafe fn as_mut_slice(&self) -> Option<&mut [u8]> {
         None
     }
@@ -288,9 +272,8 @@ pub trait GuestMemoryRegion: Bytes<MemoryRegionAddress, E = Error> {
     /// # use vm_memory::{GuestAddress, MmapRegion, GuestRegionMmap, GuestMemoryRegion};
     /// # use vm_memory::volatile_memory::{VolatileMemory, VolatileSlice, VolatileRef};
     /// #
-    /// let region = MmapRegion::<()>::new(0x400).expect("Could not create mmap region");
-    /// let region =
-    ///     GuestRegionMmap::new(region, GuestAddress(0x0)).expect("Could not create guest memory");
+    /// let region = GuestRegionMmap::<()>::from_range(GuestAddress(0x0), 0x400, None)
+    ///     .expect("Could not create guest memory");
     /// let slice = region
     ///     .as_volatile_slice()
     ///     .expect("Could not get volatile slice");
@@ -737,7 +720,7 @@ impl<T: GuestMemory + ?Sized> Bytes<GuestAddress> for T {
             buf.len(),
             addr,
             |offset, _count, caddr, region| -> Result<usize> {
-                region.write(&buf[offset as usize..], caddr)
+                region.write(&buf[offset..], caddr)
             },
         )
     }
@@ -747,7 +730,7 @@ impl<T: GuestMemory + ?Sized> Bytes<GuestAddress> for T {
             buf.len(),
             addr,
             |offset, _count, caddr, region| -> Result<usize> {
-                region.read(&mut buf[offset as usize..], caddr)
+                region.read(&mut buf[offset..], caddr)
             },
         )
     }
@@ -848,38 +831,22 @@ impl<T: GuestMemory + ?Sized> Bytes<GuestAddress> for T {
         self.try_access(count, addr, |offset, len, caddr, region| -> Result<usize> {
             // Check if something bad happened before doing unsafe things.
             assert!(offset <= count);
-            // SAFETY: Safe because we are checking the offset.
-            if let Some(dst) = unsafe { region.as_mut_slice() } {
-                // This is safe cause `start` and `len` are within the `region`, and we manually
-                // record the dirty status of the written range below.
-                let start = caddr.raw_value() as usize;
-                let end = start + len;
-                let bytes_read = loop {
-                    match src.read(&mut dst[start..end]) {
-                        Ok(n) => break n,
-                        Err(ref e) if e.kind() == std::io::ErrorKind::Interrupted => continue,
-                        Err(e) => return Err(Error::IOError(e)),
-                    }
-                };
 
-                region.bitmap().mark_dirty(start, bytes_read);
-                Ok(bytes_read)
-            } else {
-                let len = std::cmp::min(len, MAX_ACCESS_CHUNK);
-                let mut buf = vec![0u8; len].into_boxed_slice();
-                loop {
-                    match src.read(&mut buf[..]) {
-                        Ok(bytes_read) => {
-                            // We don't need to update the dirty bitmap manually here because it's
-                            // expected to be handled by the logic within the `Bytes`
-                            // implementation for the region object.
-                            let bytes_written = region.write(&buf[0..bytes_read], caddr)?;
-                            assert_eq!(bytes_written, bytes_read);
-                            break Ok(bytes_read);
-                        }
-                        Err(ref e) if e.kind() == std::io::ErrorKind::Interrupted => continue,
-                        Err(e) => break Err(Error::IOError(e)),
+            let len = std::cmp::min(len, MAX_ACCESS_CHUNK);
+            let mut buf = vec![0u8; len].into_boxed_slice();
+
+            loop {
+                match src.read(&mut buf[..]) {
+                    Ok(bytes_read) => {
+                        // We don't need to update the dirty bitmap manually here because it's
+                        // expected to be handled by the logic within the `Bytes`
+                        // implementation for the region object.
+                        let bytes_written = region.write(&buf[0..bytes_read], caddr)?;
+                        assert_eq!(bytes_written, bytes_read);
+                        break Ok(bytes_read);
                     }
+                    Err(ref e) if e.kind() == std::io::ErrorKind::Interrupted => continue,
+                    Err(e) => break Err(Error::IOError(e)),
                 }
             }
         })
@@ -936,32 +903,15 @@ impl<T: GuestMemory + ?Sized> Bytes<GuestAddress> for T {
         self.try_access(count, addr, |offset, len, caddr, region| -> Result<usize> {
             // Check if something bad happened before doing unsafe things.
             assert!(offset <= count);
-            // SAFETY: Safe because we are checking the offset is valid.
-            if let Some(src) = unsafe { region.as_slice() } {
-                // This is safe cause `start` and `len` are within the `region`.
-                let start = caddr.raw_value() as usize;
-                let end = start + len;
-                loop {
-                    // It is safe to read from volatile memory. Accessing the guest
-                    // memory as a slice should be OK as long as nothing assumes another
-                    // thread won't change what is loaded; however, we may want to introduce
-                    // VolatileRead and VolatileWrite traits in the future.
-                    match dst.write(&src[start..end]) {
-                        Ok(n) => break Ok(n),
-                        Err(ref e) if e.kind() == std::io::ErrorKind::Interrupted => continue,
-                        Err(e) => break Err(Error::IOError(e)),
-                    }
-                }
-            } else {
-                let len = std::cmp::min(len, MAX_ACCESS_CHUNK);
-                let mut buf = vec![0u8; len].into_boxed_slice();
-                let bytes_read = region.read(&mut buf, caddr)?;
-                assert_eq!(bytes_read, len);
-                // For a non-RAM region, reading could have side effects, so we
-                // must use write_all().
-                dst.write_all(&buf).map_err(Error::IOError)?;
-                Ok(len)
-            }
+
+            let len = std::cmp::min(len, MAX_ACCESS_CHUNK);
+            let mut buf = vec![0u8; len].into_boxed_slice();
+            let bytes_read = region.read(&mut buf, caddr)?;
+            assert_eq!(bytes_read, len);
+            // For a non-RAM region, reading could have side effects, so we
+            // must use write_all().
+            dst.write_all(&buf).map_err(Error::IOError)?;
+            Ok(len)
         })
     }
 
@@ -1106,6 +1056,7 @@ mod tests {
     // has the same value (and thus it did not do a non-atomic access). The test succeeds if
     // no mismatch is detected after performing accesses for a pre-determined amount of time.
     #[cfg(feature = "backend-mmap")]
+    #[cfg(not(miri))] // This test simulates a race condition between guest and vmm
     fn non_atomic_access_helper<T>()
     where
         T: ByteValued
@@ -1123,13 +1074,16 @@ mod tests {
         #[derive(Clone, Copy, Debug, Default, PartialEq)]
         struct Data<T> {
             val: T,
-            some_bytes: [u8; 7],
+            some_bytes: [u8; 8],
         }
 
         // Some sanity checks.
         assert_eq!(mem::align_of::<T>(), mem::align_of::<Data<T>>());
         assert_eq!(mem::size_of::<T>(), mem::align_of::<T>());
 
+        // There must be no padding bytes, as otherwise implementing ByteValued is UB
+        assert_eq!(mem::size_of::<Data<T>>(), mem::size_of::<T>() + 8);
+
         unsafe impl<T: ByteValued> ByteValued for Data<T> {}
 
         // Start of first guest memory region.
@@ -1148,7 +1102,7 @@ mod tests {
         // Need to clone this and move it into the new thread we create.
         let mem2 = mem.clone();
         // Just some bytes.
-        let some_bytes = [1u8, 2, 4, 16, 32, 64, 128];
+        let some_bytes = [1u8, 2, 4, 16, 32, 64, 128, 255];
 
         let mut data = Data {
             val: T::from(0u8),
@@ -1195,6 +1149,7 @@ mod tests {
 
     #[cfg(feature = "backend-mmap")]
     #[test]
+    #[cfg(not(miri))]
     fn test_non_atomic_access() {
         non_atomic_access_helper::<u16>()
     }
