@@ -10,8 +10,6 @@
 
 //! Helper structure for working with mmaped memory regions in Unix.
 
-use std::error;
-use std::fmt;
 use std::io;
 use std::os::unix::io::AsRawFd;
 use std::ptr::null_mut;
@@ -19,59 +17,38 @@ use std::result;
 
 use crate::bitmap::{Bitmap, BS};
 use crate::guest_memory::FileOffset;
-use crate::mmap::{check_file_offset, AsSlice, NewBitmap};
-use crate::volatile_memory::{self, compute_offset, VolatileMemory, VolatileSlice};
+use crate::mmap::{check_file_offset, NewBitmap};
+use crate::volatile_memory::{self, VolatileMemory, VolatileSlice};
 
 /// Error conditions that may arise when creating a new `MmapRegion` object.
-#[derive(Debug)]
+#[derive(Debug, thiserror::Error)]
 pub enum Error {
     /// The specified file offset and length cause overflow when added.
+    #[error("The specified file offset and length cause overflow when added")]
     InvalidOffsetLength,
     /// The specified pointer to the mapping is not page-aligned.
+    #[error("The specified pointer to the mapping is not page-aligned")]
     InvalidPointer,
     /// The forbidden `MAP_FIXED` flag was specified.
+    #[error("The forbidden `MAP_FIXED` flag was specified")]
     MapFixed,
     /// Mappings using the same fd overlap in terms of file offset and length.
+    #[error("Mappings using the same fd overlap in terms of file offset and length")]
     MappingOverlap,
     /// A mapping with offset + length > EOF was attempted.
+    #[error("The specified file offset and length is greater then file length")]
     MappingPastEof,
     /// The `mmap` call returned an error.
+    #[error("{0}")]
     Mmap(io::Error),
     /// Seeking the end of the file returned an error.
+    #[error("Error seeking the end of the file: {0}")]
     SeekEnd(io::Error),
     /// Seeking the start of the file returned an error.
+    #[error("Error seeking the start of the file: {0}")]
     SeekStart(io::Error),
 }
 
-impl fmt::Display for Error {
-    fn fmt(&self, f: &mut std::fmt::Formatter) -> fmt::Result {
-        match self {
-            Error::InvalidOffsetLength => write!(
-                f,
-                "The specified file offset and length cause overflow when added"
-            ),
-            Error::InvalidPointer => write!(
-                f,
-                "The specified pointer to the mapping is not page-aligned",
-            ),
-            Error::MapFixed => write!(f, "The forbidden `MAP_FIXED` flag was specified"),
-            Error::MappingOverlap => write!(
-                f,
-                "Mappings using the same fd overlap in terms of file offset and length"
-            ),
-            Error::MappingPastEof => write!(
-                f,
-                "The specified file offset and length is greater then file length"
-            ),
-            Error::Mmap(error) => write!(f, "{}", error),
-            Error::SeekEnd(error) => write!(f, "Error seeking the end of the file: {}", error),
-            Error::SeekStart(error) => write!(f, "Error seeking the start of the file: {}", error),
-        }
-    }
-}
-
-impl error::Error for Error {}
-
 pub type Result<T> = result::Result<T, Error>;
 
 /// A factory struct to build `MmapRegion` objects.
@@ -165,6 +142,7 @@ impl<B: Bitmap> MmapRegionBuilder<B> {
             (-1, 0)
         };
 
+        #[cfg(not(miri))]
         // SAFETY: This is safe because we're not allowing MAP_FIXED, and invalid parameters
         // cannot break Rust safety guarantees (things may change if we're mapping /dev/mem or
         // some wacky file).
@@ -179,10 +157,22 @@ impl<B: Bitmap> MmapRegionBuilder<B> {
             )
         };
 
+        #[cfg(not(miri))]
         if addr == libc::MAP_FAILED {
             return Err(Error::Mmap(io::Error::last_os_error()));
         }
 
+        #[cfg(miri)]
+        if self.size == 0 {
+            return Err(Error::Mmap(io::Error::from_raw_os_error(libc::EINVAL)));
+        }
+
+        // Miri does not support the mmap syscall, so we use rust's allocator for miri tests
+        #[cfg(miri)]
+        let addr = unsafe {
+            std::alloc::alloc_zeroed(std::alloc::Layout::from_size_align(self.size, 8).unwrap())
+        };
+
         Ok(MmapRegion {
             addr: addr as *mut u8,
             size: self.size,
@@ -401,21 +391,6 @@ impl<B: Bitmap> MmapRegion<B> {
     }
 }
 
-impl<B> AsSlice for MmapRegion<B> {
-    // SAFETY: This is safe because we mapped the area at addr ourselves, so this slice will not
-    // overflow. However, it is possible to alias.
-    unsafe fn as_slice(&self) -> &[u8] {
-        std::slice::from_raw_parts(self.addr, self.size)
-    }
-
-    // SAFETY: This is safe because we mapped the area at addr ourselves, so this slice will not
-    // overflow. However, it is possible to alias.
-    #[allow(clippy::mut_from_ref)]
-    unsafe fn as_mut_slice(&self) -> &mut [u8] {
-        std::slice::from_raw_parts_mut(self.addr, self.size)
-    }
-}
-
 impl<B: Bitmap> VolatileMemory for MmapRegion<B> {
     type B = B;
 
@@ -428,19 +403,17 @@ impl<B: Bitmap> VolatileMemory for MmapRegion<B> {
         offset: usize,
         count: usize,
     ) -> volatile_memory::Result<VolatileSlice<BS<B>>> {
-        let end = compute_offset(offset, count)?;
-        if end > self.size {
-            return Err(volatile_memory::Error::OutOfBounds { addr: end });
-        }
+        let _ = self.compute_end_offset(offset, count)?;
 
         Ok(
             // SAFETY: Safe because we checked that offset + count was within our range and we only
             // ever hand out volatile accessors.
             unsafe {
                 VolatileSlice::with_bitmap(
-                    (self.addr as usize + offset) as *mut _,
+                    self.addr.add(offset),
                     count,
                     self.bitmap.slice_at(offset),
+                    None,
                 )
             },
         )
@@ -453,7 +426,14 @@ impl<B> Drop for MmapRegion<B> {
             // SAFETY: This is safe because we mmap the area at addr ourselves, and nobody
             // else is holding a reference to it.
             unsafe {
+                #[cfg(not(miri))]
                 libc::munmap(self.addr as *mut libc::c_void, self.size);
+
+                #[cfg(miri)]
+                std::alloc::dealloc(
+                    self.addr,
+                    std::alloc::Layout::from_size_align(self.size, 8).unwrap(),
+                );
             }
         }
     }
@@ -540,6 +520,7 @@ mod tests {
     }
 
     #[test]
+    #[cfg(not(miri))] // Miri cannot mmap files
     fn test_mmap_region_from_file() {
         let mut f = TempFile::new().unwrap().into_file();
         let offset: usize = 0;
@@ -558,6 +539,7 @@ mod tests {
     }
 
     #[test]
+    #[cfg(not(miri))] // Miri cannot mmap files
     fn test_mmap_region_build() {
         let a = Arc::new(TempFile::new().unwrap().into_file());
 
@@ -610,7 +592,7 @@ mod tests {
             MmapRegion::build(Some(FileOffset::from_arc(a, offset)), size, prot, flags).unwrap();
 
         assert_eq!(r.size(), size);
-        assert_eq!(r.file_offset().unwrap().start(), offset as u64);
+        assert_eq!(r.file_offset().unwrap().start(), offset);
         assert_eq!(r.prot(), libc::PROT_READ | libc::PROT_WRITE);
         assert_eq!(r.flags(), libc::MAP_NORESERVE | libc::MAP_PRIVATE);
         assert!(r.owned());
@@ -628,6 +610,7 @@ mod tests {
     }
 
     #[test]
+    #[cfg(not(miri))] // Causes warnings due to the pointer casts
     fn test_mmap_region_build_raw() {
         let addr = 0;
         let size = unsafe { libc::sysconf(libc::_SC_PAGESIZE) as usize };
@@ -646,6 +629,7 @@ mod tests {
     }
 
     #[test]
+    #[cfg(not(miri))] // Miri cannot mmap files
     fn test_mmap_region_fds_overlap() {
         let a = Arc::new(TempFile::new().unwrap().into_file());
         assert_eq!(unsafe { libc::ftruncate(a.as_raw_fd(), 1024 * 10) }, 0);
